// See COPYING for license information.

#include <cstdlib>
#include <algorithm>
#include <fstream>
#include "defs.h"
#include "CConfigFile.h"
#include "CParser.h"

CParser::CParser()
{
	// Parse installed.
	std::ifstream	_logfile;
	std::string	_inputstr;
	std::cout << "Parsing installed packages...";
	std::cout.flush();
	system(std::string("ls /var/log/packages > " + TMPFILE("installedpackages.log")).c_str());
	_logfile.open(TMPFILE("installedpackages.log").c_str(), std::ios_base::in);
 	while ( !_logfile.eof() ) {
		getline(_logfile, _inputstr);
		if ( _inputstr.length() > 0 ) parse_file("/var/log/packages/" + _inputstr);
	}
	_logfile.close();
	std::cout << "done\n";

	// Parse the mirror package list.
	std::cout << "Parsing mirror...";
	std::cout.flush();
	if ( !parse_file(TMPFILE("PACKAGES.TXT")) ) {
		std::string	_curmirror = CConfigFile::Instance().GetValue("CurrentMirror")[0];
		if ( _curmirror.length() > 8 )	parse_search(_curmirror.substr(8));
	}
	std::cout << "done\n";
}

void CParser::parse_search(const std::string& directory)
{	// Search a directory tree for packages, for when there is no PACKAGES.TXT.
	system(std::string("find '" + directory + "' -regex \".*\\.t[glbx]z\" > " + TMPFILE("localmirrorsearch.tmp")).c_str());
	std::ifstream	_ifile;
	std::string	_instr;
	_ifile.open(TMPFILE("localmirrorsearch.tmp").c_str(), std::ios_base::in);
	while ( !_ifile.eof() && _ifile.is_open() ) {
		getline(_ifile, _instr);
		if ( !_instr.length() ) break;
		std::string	_pkg = get_package_name(_instr.substr(_instr.rfind('/') + 1));
		std::transform(_pkg.begin(), _pkg.end(), _pkg.begin(), tolower);

		Packages[_pkg].name = get_package_name(_instr.substr(_instr.rfind('/') + 1));
		Packages[_pkg].mirror_version = get_package_version(_instr.substr(_instr.rfind('/') + 1));
		Packages[_pkg].location = _instr.substr(directory.length() + 1);
		if ( (int)Packages[_pkg].location.rfind('/') >= 0 )
			Packages[_pkg].location.erase(Packages[_pkg].location.rfind('/'));
		else	Packages[_pkg].location = "";
	}
}

bool CParser::parse_file(const std::string& filename)
{	// Grab minimalistic information from a larger package DB.
	std::ifstream	_iflist;
	std::string	_instr;
	std::string	_pkg;
	_iflist.open(filename.c_str(), std::ios_base::in);
	if ( !_iflist.is_open() )	return false;
	while ( _iflist.is_open() && !_iflist.eof() ) {
		getline(_iflist, _instr);
		if ( !_instr.find("PACKAGE NAME:") ) {
			_instr = trim(_instr.substr(13));
			_pkg = get_package_name(_instr);
			if ( !_pkg.length() )	continue;	// Name is blank, moving right along.
			std::transform(_pkg.begin(), _pkg.end(), _pkg.begin(), tolower);

			Packages[_pkg].name = get_package_name(_instr);
			if ( !filename.find("/var/log")	)	// Installed package.
				Packages[_pkg].installed_version = get_package_version(_instr + ".aaa");
			else {	// Mirror package.
				Packages[_pkg].mirror_version = get_package_version(_instr);
				Packages[_pkg].type = _instr.substr(_instr.rfind('.'));
			}
		} else if ( !_instr.find("PACKAGE LOCATION:") && filename.find("/var/log") ) {
			Packages[_pkg].location = trim(_instr.substr(17));
			if ( !Packages[_pkg].location.find("./") )
				Packages[_pkg].location.erase(0, 2);
		} else if ( !_instr.find("PACKAGE REQUIRED:") )	// Dependencies.
			Packages[_pkg].dependencies = "," + trim(_instr.substr(17)) + ", ";
	}
	_iflist.close();
	return true;
}

std::string CParser::get_package_name(const std::string& fromstr)
{	// Returns just the package name from an entire string, i.e. "fghfgh" from "fghfgh-10.0.1-i486-62.1.tgz"
	int	_dashcount = 0;
	for ( int i = fromstr.length() - 1; i >= 0; i-- ) {
		if ( fromstr.substr(i, 1) == "-" ) {
			_dashcount++;
			if ( _dashcount < 3 ) continue;
			return fromstr.substr(0, i);
		}
	}
	if ( _dashcount > 0 ) return fromstr.substr(0, fromstr.find('-'));	// Helps out screwed up packages.
	return "null";	// Oh noez!
}

std::string CParser::get_package_version(const std::string& fromstr)
{	// Returns just the version from the package filename, i.e. "10.0.1-i486-62.1" from "fghfgh-10.0.1-i486-62.1.tgz"
	int	_dashcount = 0;
	for ( int i = fromstr.length() - 1; i >= 0; i-- ) {
		if ( fromstr.substr(i, 1) == "-" ) {
			_dashcount++;
			if ( _dashcount == 3 )
				return fromstr.substr(i + 1, (fromstr.length() - i) - 5);
		}
	}
	// Fix for messed up packages like cupswrapperMFC that Atticus pointed out.
	if ( _dashcount > 0 ) return fromstr.substr(fromstr.find('-') + 1, (fromstr.length() - fromstr.find('-')) - 5);
	return "null";	// Hopefully this is never called.
}

std::string CParser::trim(std::string str)
{	// This function trims spaces off the ends of a string.
	while ( str.length() > 0 ) {
		if ( str.substr(str.length() - 1) == " " )	str.erase(str.length() - 1);
		else if ( !str.find(' ') )	str.erase(0, 1);
		else break;
	}
	return str;
}
